package ovh.gasser.newshapes.ui;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import ovh.gasser.newshapes.HTMLExporter;
import ovh.gasser.newshapes.Selection;
import ovh.gasser.newshapes.attributes.ColorAttributes;
import ovh.gasser.newshapes.shapes.SCollection;
import ovh.gasser.newshapes.shapes.Shape;

import java.awt.*;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.FileNotFoundException;
import java.util.Optional;

public class Controller {
    private final static Logger logger = LoggerFactory.getLogger(Controller.class);
    private final ShapesView view;
    private final SCollection model;
    private Selection selection;

    Controller(ShapesView view, SCollection model) {
        this.view = view;
        this.model = model;

        var adapter = new MouseAdapter() {
            @Override
            public void mousePressed(MouseEvent evt) {
                handleMousePressed(evt);
            }

            @Override
            public void mouseDragged(MouseEvent evt) {
                handleMouseDragged(evt);
            }
        };
        this.view.addMouseMotionListener(adapter);
        this.view.addMouseListener(adapter);

        this.view.setFocusable(true);
        boolean res = this.view.requestFocusInWindow();
        assert res;
        this.view.addKeyListener(new KeyAdapter() {
            @Override
            public void keyPressed(KeyEvent e) {
                handleKeyPressed(e);
            }
        });
    }

    private void handleMouseDragged(MouseEvent evt) {
        if (selection != null) selection.shape.setLoc(evt.getPoint());
        view.repaint();
    }

    private void handleMousePressed(MouseEvent evt) {
        getTarget(evt, this.model)
                .ifPresentOrElse(
                        s -> {
                            if (selection != null) resetSelection();
                            selection = new Selection(s, true);
                            logger.debug("Selecting {}", selection.shape);
                        },
                        () -> {
                            if (selection != null) resetSelection();
                        }
                );
        view.repaint();
    }

    private void handleKeyPressed(KeyEvent evt) {
        switch (evt.getKeyCode()) {
            case KeyEvent.VK_DELETE -> deleteSelected();
            case KeyEvent.VK_C -> copySelection();
            case KeyEvent.VK_A -> changeSelectionColor();
            case KeyEvent.VK_H -> exportHtml();
            default -> logger.warn("Pressed unhandled key: {}", evt.getKeyChar());
        }
    }

    private void exportHtml() {
        logger.info("Exporting view to html");
        try {
            new HTMLExporter(this.model).export();
        } catch (FileNotFoundException e) {
            logger.error("Unable to export html: {}", e.getMessage());
        }
    }

    private void changeSelectionColor(){
        if (selection == null) {
            logger.debug("No selection to change color of");
            return;
        }
        if (selection.shape instanceof SCollection collection) {
            collection.forEach(shape -> shape.addAttributes(new ColorAttributes(false, true, Color.BLACK, new Color((int) (Math.random() * 0x1000000)))));
        } else {
            selection.shape.addAttributes(new ColorAttributes(false, true, Color.BLACK, new Color((int) (Math.random() * 0x1000000))));
        }
        view.repaint();
    }

    private void copySelection() {
        if (selection == null) {
            logger.debug("No selection to copy");
            return;
        }
        this.model.add(selection.shape.clone());
        view.repaint();
    }

    private void deleteSelected() {
        if (selection == null) return;
        logger.debug("Deleting selected shape(s)");
        this.model.remove(selection.shape);
        resetSelection();
        view.repaint();
    }

    private void resetSelection() {
        logger.debug("Un-selecting {}", selection.shape);
        selection.unselect();
        selection = null;
    }

    private Optional<Shape> getTarget(MouseEvent evt, SCollection sc) {
        return sc.stream()
                .filter(s -> s.getBounds().contains(evt.getPoint()))
                .findFirst();
    }

}
